"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
function TaggedTemplateExpression(path, { opts }) {
    const scope = path.scope;
    const customClassName = Array.isArray(opts.className)
        ? opts.className.join(' ')
        : opts.className;
    try {
        // simple case
        if (isStyledPrefix(path.node.tag)) {
            if (!customClassName)
                return;
            path.node.tag = insertBefore(path.node.tag, customClassName);
            return;
        }
        // chained case.  traverse until prefix found.
        // NB: styled-component chain api is always CallExpression+MemberExpression pairs.
        let node = path.node.tag;
        while (true) {
            if (!node || !node.callee)
                break;
            if (isStyledPrefix(node.callee.object)) {
                if (!customClassName)
                    return;
                node.callee.object = insertBefore(node.callee.object, customClassName);
                break;
            }
            node = node.callee.object;
        }
    }
    catch (e) { }
    // hoisted helpers in closure
    function insertBefore(node, className) {
        return core_1.types.callExpression(core_1.types.memberExpression(node, core_1.types.identifier('attrs')), [
            core_1.types.objectExpression([
                core_1.types.objectProperty(core_1.types.stringLiteral('className'), core_1.types.stringLiteral(className)),
            ]),
        ]);
    }
    function isStyledPrefix(node) {
        // handle two forms: styled.div and styled(Comp)
        return ((core_1.types.isMemberExpression(node) && isStyledComponentsBinding(node.object)) ||
            (core_1.types.isCallExpression(node) && isStyledComponentsBinding(node.callee)));
        function isStyledComponentsBinding(node) {
            if (!core_1.types.isIdentifier(node))
                return false;
            const binding = scope.getBinding(node.name);
            if (!binding || binding.kind !== 'module')
                return false;
            return binding.path.parent.source.value === 'styled-components';
        }
    }
}
exports.default = TaggedTemplateExpression;
